;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Koopatrol from Paper Mario
;
;Uses first extra bit: YES
;
;If the first extra bit is set, this sprite will end up charging at you faster,
;and spinning at you in its shell faster. It will also have a different palette.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;DEFINES
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

			!SHELLTIMER					= $15AC
			!STATE						= $C2
			!FRAMES						= $1570
			!DISABLESHELLTIMER			= $163E
			!MAXSPEED					= $1510

			!SIGHT						= $40
			
			!TEMP = $09

			!SLOTPTR = $0660					;16bit pointer for source GFX
			!SLOTBANK = $0662					;bank
			!SLOTDEST = $0663					;VRAM address
			!SLOTSUSED = $06FE					;how many slots have been used

			!MAXSLOTS = $04						;maximum selected slots

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;SPEED TABLES
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;			

XSPEEDTABLE:
			db $10,$F0							;		max X-speeds when walking
			
XACCELTABLE:
			db $01,$FF							;		acceleration speeds when walking
			
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;		
			
XSPEEDTABLESC:
			db $38,$C8							;		max X-speeds when charging at you
			
XSPEEDTABLESA:
			db $04,$FC							;		acceleration speeds when charging at you	
			
XSPEEDTABLEHIT:
			db $F8,$08							;		X-speeds to use when bouncing off a wall	

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;			

XSPEEDTABLESS:
			db $40,$C0							;		max X-speeds when shell spinning

XSPEEDTABLESAC:
			db $04,$FC							;		acceleration speeds when shell spinning	
			
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; INIT ROUTINE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

			print "INIT ",pc
			
			JSR SUB_HORZ_POS					;\	
			TYA									; | 	transfer direction
			STA $157C,x							;/
			RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; MAIN ROUTINE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

			print "MAIN ",pc

			PHB		        					;\	
			PHK									; | 	Change the data bank to the one our code is running from.
			PLB									; | 	so nothing glitches out at all

			JSR SpriteCode						; | 	Jump to the sprite's function.
			
			PLB									; | 	Restore old data bank.
			RTL									;/		And return.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Sprite Code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ReturnS:
			RTS
			
KILLD:
			LDA $140D							;\		if spinjumping
			ORA $63								; | 	or possibly ground pounding
			BEQ NoSpin							;/		run this code
												;		else, skip it
			
			LDA $74								; \ 	skip y-speed if climbing 
			BNE +								; / 

			LDA #$FE							;\		load speed if A or B is not pressed
			BIT $15								; |		do a bit check on $15 without altering A
			BPL ++								;/		if bit 7 is clear, meaning you're not holding A, store already

			LDA #$F8							;		else, go down "slower"

++    
			STA $7D								;		store to mario's yspeed

+				
			JSR SUB_STOMP_PTS
			JMP Die								;		jump to the die code
			
NoSpin:											;\		or, if mario's not spinning,
			JSL $01AA33							; |		boost him up like normal
			JSL $01AB99							; |		display the contact graphic
			JSR SUB_STOMP_PTS					; |		give mario points
			
			LDA #$0F							; | 	and set the sprite state to: "about to die"
			STA !STATE,x						; |
			
			STZ !FRAMES,x						; |		zero out the koopatrol's frames
			
			LDA #$40							; |		so the GFX doesn't glitch out
			STA !SHELLTIMER,x					; |		and set how long the have him show the "flailing" animation
			RTS									;/		return

ws:
			LDA #$E0							;\		store to the sprite's Y-speed
			STA $AA,x							;/
			
			LDY $157C,x							;		load sprite direction into Y
			
			LDA XSPEEDTABLEHIT,y				;\		use a table
			STA $B6,x							;/		to determine the sprite's X-speed

			JSR HAS_STAR						;		points subroutine
			
			LDA #$0E							;\		set state to
			STA !STATE,x						;/		falling over
			
			STZ !FRAMES,x						;		zero out current frame
			
			LDA #$0C							;\		and set timer to change to next state
			STA !SHELLTIMER,x					;/
			
			JSL $01802A							;		update positions with gravity			
			
			RTS									;		return
			
SpriteCode:
			JSR SUB_OFF_SCREEN_X0				;\		run sprite offscreen checks
			
			JSR SUB_GFX							; |		run the graphics code
		
			LDA $14C8,x							; |		if sprite state != 8,
			CMP #$08							; |		return
			BNE ReturnS							;/		to previous return, as BNE is out of range
			
			LDA $9D								;\		if sprites are locked,
			BNE ReturnS							;/		return
			
			LDA !STATE,x						;\		if sprite state is 0F
			CMP #$0E							; |		meaning that the koopatrol is about to die,
			BCS sdfat							;/		skip interaction code
			
			JSL $01A7DC							;\		check for mario/sprite interaction
			BCC sdfat							; |		carry is set if mario is touching the sprite
												;/		if carry is clear, return
			
			LDA $1490
			BNE ws
			
			LDA $0E								;\		if mario touching side of sprite,
			CMP #$E6							; |
			BPL sdfad							; |		hurt him
			
			LDA !STATE,x						; |		if the koopatrol is in its shell,
			CMP #$03							; |
			BEQ sfsae							; |		allow mario to jump on it, both ways
			
			CMP #$07							; |		if it just recently hit a wall by charging,
			BNE noKILLD							; |		go kill him
			
			JMP KILLD
			
noKILLD:			
			LDA $140D							; |		if mario is NOT spin jumping,
			BEQ sdfad							; |		hurt him

sfsae:
			JSL $01AB99							; |		boost mario's speed
			JSL $01AA33							; |		display contact graphic
			
			LDA #$02							; |		play a sound
			STA $1DF9							; |
			
			BRA sdfat							;/		and continue on
			
sdfad:			
			JSL $00F5B7							;		hurt mario
			
sdfat:			
			JSL $019138							;\		run object interaction code		
			
			LDA !STATE,x						; |		disable any value >= 0F
			AND #$0F							; |		so nothing glitches up
			STA !STATE,x						;/		and store it back
			
			PER $0023
			
			JSL $0086DF							;		jsl to the game's absolute pointer code
			
KoopatrolPtrs:
			dw Idle								;\		idle code
			dw Walk								; |		walking code
			dw GoingInShell						; |		about to go in shell
			dw ShellSpin						; |		actually spinning in the shell
			dw GettingOutOfShell				; |		getting out of the shell
			dw Charge							; |		charging at mario
			dw HitWall							; |		just hit the wall
			dw Flail							; |		and is now on its shell, flailing
			dw GetUp							; |		getting back up
			dw Filler							; |		ignore
			dw Filler							; |		ignore
			dw Filler							; |		ignore
			dw Filler							; |		ignore
			dw Filler							; |		ignore
			dw Falling							; |		falling
			dw Dead								;/		dying
			
Return:
			JSR SPRITE_INTERACT

Filler:
			RTS									;		return from sprite code
			
Dead:
			LDA $1588,x							;\		if on the ground,
			AND #$04							; |
			BNE +								;/		do not update positions
			
			JSL $01802A							;		update positions with gravity	

+			
			LDA !SHELLTIMER,x					;\		if the death timer hasn't run out yet...
			BEQ Die								; |		keep animating
			
			LDA $14								; |		check if timer is an interval of 4
			AND #$03							; |		if not, skip updating the image
			BNE DontUpdateImgD					; |
			
			INC !FRAMES,x						; |		update the image...

			LDA !FRAMES,x						; |		and fix the counter
			AND #$03							; |
			STA !FRAMES,x						; |
			
DontUpdateImgD:			
			RTS									;/		return
			
Die:
			LDA #$04							;\		set the sprite state to: killed by spin jump
			STA $14C8,x							; |

			LDA #$2E							; |		set the sprite spin jump death counter
			STA $1540,x							; |
			
			LDA #$08							; |		play the spin jump sound
			STA $1DF9							; |
			
			JSL $07FC3B							; |		and run the spin jump star code
			RTS									;/		routine
			
Falling:
			JSL $01802A							;		update positions with gravity	

			LDA !SHELLTIMER,x					;\		if not time to change state,
			BEQ ChangeStateFL					;/		don't branch

			LDA $14								;\		if frame counter is not an interval of 4,
			AND #$03							; |		don't update image
			BNE DontUpdateImgFL					;/
			
			INC !FRAMES,x						;		increment current frame
			
DontUpdateImgFL:
			RTS									;		return
			
ChangeStateFL:
			INC !STATE,x						;		increment state to dying
			
			STZ !FRAMES,x						;		zero out current frame
			
			LDA #$40							;\		load how long to flail
			STA !SHELLTIMER,x					;/		into a timer
			
			RTS									;		return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GRAPHICS ROUTINE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Properties:
			db $41,$01							;		properties for direction

Tiles:
			db $00,$02,$20,$22					;		tiles to use (DO NOT CHANGE AT ALL)
XOffset:
			db $FC,$0C,$FC,$0C					;		X-offset for the 4 16x16 tiles facing left

XOffset2:
			db $04,$F4,$04,$F4					;		the X-offset fot the 4 16x16 tiles when facing right
			
YOffset:
			db $EB,$EB,$FB,$FB					;		normal Y-offset
					
IdleFrames:
			db $00,$01,$02,$03					;\		list of idle frames
			db $02,$01,$00,$01					; |
			db $02,$03,$02,$01					;/
			
WalkFrames:
			db $01,$61,$62,$61					;\		list of walking frames
			db $01,$63,$70,$63					;/

GoingInShellFrames:
			db $60,$53,$52,$51					;		list of going in shell frames
			
SpinningShellFrames:
			db $33,$40,$41,$42					;\		list of spinning shell frames
			db $43,$50,$33,$40					; |
			db $41,$42,$43,$50					;/

GettingOutOfShellFrames:
			db $51,$52,$53,$60					;		list of getting out of shell frames
			
ChargingFrames:
			db $30,$31,$32,$31					;		list of charging frames
			
HitWallFrames:
			db $11,$12,$13,$21					;		list of the "hit wall" frames
			
FlailFrames:
			db $22,$23,$22,$21					;		list of flailing frames
			
GettingUpFrames:
			db $71,$72,$73,$80					;\		list of getting up frames
			db $81,$82							;/

SUB_GFX:
           	JSR GET_DRAW_INFO					; after: 	Y = index to sprite OAM ($300)
												; 			$00 = sprite x position relative to screen boarder 
												; 			$01 = sprite y position relative to screen boarder  
            
			LDA $157C,x							;\		store sprite direction
			STA $08								; |		to temporary RAM address
			
			STZ $06								; |		zero out tiles drawn
			
			LDA !FRAMES,x						; |		load the current frame number
			
			PHX									; |		save sprite index
			TAX									; |		transfer frame number to X

			PHX									; |		but first, temporarily load sprite index again
			LDX $15E9							;/

			LDA !STATE,x						;\		to tell which frame table to load
			PLX									; |		pull back X
			CMP #$01							; |		if sprite state is 1, walking
			BEQ Walking							;/
			
			CMP #$02							;\		if sprite state is 2, going in shell
			BEQ InShell							;/
			
			CMP #$03							;\		if sprite state is 3, spinning inside of shell
			BEQ SpinningShell					;/
			
			CMP #$04							;\		if sprite state is 4, getting out of shell
			BEQ OutShell						;/
			
			CMP #$05							;\		if sprite state is 5, charging at mario
			BEQ Charging						;/
			
			CMP #$06							;\		if sprite state is 6, hit the wall
			BEQ HitTheWall						;/
			
			CMP #$07							;\		if sprite state is 7, flailing
			BEQ Flailing						;/		
			
			CMP #$08							;\		if sprite state is 8, getting up
			BEQ GettingUp						;/
			
			CMP #$0E							;\		if sprite state is E, hit the wall
			BEQ HitTheWall						;/
			
			CMP #$0F							;\		if sprite state is F, show the same frames as if it were flailing
			BEQ Flailing						;/
			
			LDA IdleFrames,x					;\		else, load the normal idle frames
			BRA SST								;/
			
InShell:
			LDA GoingInShellFrames,x			;\		load frames	
			BRA SST								;/
			
SpinningShell:
			LDA SpinningShellFrames,x			;\		load frames
			BRA SST								;/

OutShell:
			LDA GettingOutOfShellFrames,x		;\		load frames
			BRA SST								;/
			
Charging:
			LDA ChargingFrames,x				;\		load frames
			BRA SST								;/

HitTheWall:
			LDA HitWallFrames,x					;\		load frames
			BRA SST								;/		

Flailing:
			LDA FlailFrames,x					;\		load frames
			BRA SST								;/

GettingUp:
			LDA GettingUpFrames,x				;\		load frames
			BRA SST								;/
			
Walking:
			LDA WalkFrames,x					;\		load frames
			
SST:				
			PLX									;/		pull X back
			JSR GETSLOT							; 		...and run the dynamic sprite routine
			BNE +								;\		if no slot is found,
			JMP ReturnGFX						; |		return (BEQ is out of range)
			
+			
			STA $07								;/		store the dynamic frame slot to $07
			
			PHX									;		push sprite index...
			
			LDX #$03							;\		and set up a tile loop
			
TileLoop:
			LDA $08								; |		if facing right,
			BEQ ++								;/		use a different table

			LDA $00								;\		take the X-position of GET_DRAW_INFO
			CLC									; |
			ADC XOffset,x						; |		and add in the X-offsets
			STA $0300,y							;/		store to the X-position
			BRA +++
			
++
			LDA $00								;\		same
			CLC									; |
			ADC XOffset2,x						; |		but use a different table
			STA $0300,y							;/		store to the X-position

+++			
			LDA $01								;\		take the output of GET_DRAW_INFO
			CLC									; |	
			ADC YOffset,x						; |		add in Y-displacement when needed
			ADC #$04							; |		add another 4
			STA $0301,y							;/		and store to the Y-position
					
			LDA $07								;\		take the slot number from before,
			CLC									; |
			ADC Tiles,x							; |		add in the tile number
			STA $0302,y							;/		and store it
					
			PHX									;\		load sprite index again to get palette info
			LDX $15E9							;/
			
			LDA $7FAB10,x						;\		if the first extra bit is set,
			AND #$04							; |		then run this below code
			BEQ +								;/		to change the palette
			
			LDA $15F6,x							;\		get regular ccct values
			ORA #$0F							; |		ORA #$0F to change palette to 0F
			BRA ++								;/		and skip the below code so A is not overwritten
			
+
			LDA $15F6,x							;		get ccct values

++				
			LDX $08								; |		change direction of the tile based on the sprite direction
			ORA Properties,x					; |		add in the x flip, if needed
			ORA $64								; |		add in level stuff
			STA $0303,y							;/		and store to the properties
			
           		PLX									;		pull X back
			
			INC $06								;		increment the number of frames drawn,
			
			INY #4								;		increase sprite index to OAM,
			
			DEX									;\		and then decrease X
			BPL TileLoop						;/		if X is not FF, go back and keep running
			
			PLX									;		pull the sprite index back

			LDA !STATE,x						;\		if sprite state is 2,
			CMP #$02							; |		draw some extra tiles if it's about to go in the shell
			BEQ InShellI						;/		since it doesn't fit completely in a 32x32 region
			
			CMP #$04							;\		if sprite state is 4,
			BEQ InShellI						;/		do the same thing as above
			
			CMP #$05							;\		if sprite state is 5,
			BEQ RunI							;/		draw the extra tiles for the feet
			
			CMP #$06							;\		if sprite state is 6,
			BEQ HitI							;/		draw the extra tiles for the spike on its head
			
			CMP #$07							;\		if sprite state is 7,
			BEQ FlailingI						;/		draw the extra tiles for the rotated spike on its head
			
			CMP #$08							;\		if sprite state is 8,
			BEQ GetUpI							;/		draw the extra tiles for the spike on its head

			CMP #$0E							;\		if sprite state is E,
			BEQ HitI							;/		draw the extra tiles for the spike on its head
			
			CMP #$0F							;\		if sprite state is F,
			BEQ FlailingI						;/		draw the extra tiles for the rotated spike on its head
			
			JSR IDLE_EXTRA						;\		if previous statements are not met,
			BRA SF								;/		draw the extra tiles for the spike on its head when idling or walking

InShellI:
			JSR SHELL_EXTRA						;\		draw tiles
			BRA SF								;/
			
RunI:
			JSR RUN_EXTRA						;\		draw tiles
			BRA SF								;/		

HitI:
			JSR HIT_EXTRA						;\		draw tiles
			BRA SF								;/		

GetUpI:
			JSR GET_UP_EXTRA					;\		draw tiles
			BRA SF								;/	
			
FlailingI:
			JSR FLAIL_EXTRA						;		draw tiles

SF:			
			LDY #$02							;\		Y = 02 because we drew 16x16s
			LDA $06								; |		A = 06, without decrementing as we drew another tile
												;/		if we didn't draw any, stop drawing
			BEQ ReturnGFX
			JSL $01B7B3							;		draw the actual tiles!
			
ReturnGFX:
			RTS									;		and return

YDISPLACEMENTS:
			db $E8,$E9,$EA,$EB					;\		Y-displacements
			db $EA,$E9,$E8,$E9					; |		etc
			db $EA,$EB,$EA,$E9					;/		etc
			
YDISPLACEMENTSWALK:
			db $E9,$EA,$EC,$EA					;\		Y-displacements when walking
			db $E9,$EA,$EB,$EA					;/

IDLE_EXTRA:
			LDA !STATE,x						;\		if sprite is actually in its shell, SKIP THIS
			CMP #$03							; |		and decrement $06, as it is the number of tiles drawn
			BEQ asdf							;/

			LDA !FRAMES,x						;\		load frame counter
			TAX									;/		and transfer it to X

			LDA $00								;\		take the X-position from GET_DRAW_INFO
			STA $0300,y							;/		and store it to the X-position
			
			PHX									;\		load sprite index temporarily, again
			LDX $15E9							; |
			
			LDA !STATE,x						; |		if the sprite is walking,
			PLX									; |
			CMP #$01							; |		go use the other Y-displacement table
			BEQ Walkingg						;/
			
			LDA $01								;\		take the Y-position from GET_DRAW_INFO
			CLC									; |
			ADC YDISPLACEMENTS,x				; |		add in Y-displacement...
			STA $0301,y							;/		and store it to the Y-position
			BRA AAA								;		skip next code
			
Walkingg:
			LDA $01								;\		take the Y-position from GET_DRAW_INFO
			CLC									; |
			ADC YDISPLACEMENTSWALK,x			; |		add in Y-displacements...
			STA $0301,y							;/		and store it to the Y-position

AAA:			
			LDA #$A4							;\		store the tile number
			STA $0302,y							;/
			
			LDX $15E9							;		load sprite index, again
			
			LDA $15F6,x							;		get ccct values
			
			PHX									;\		preserve X,
			LDX $08								; |		load sprite direction
			ORA Properties,x					; |		and apply X-flip if needed
			PLX									;/		pull back X
			
			ORA $64								;\		add in level stuff
			STA $0303,y							;/		and store the properties
			
			RTS									;		return
			
asdf:
			DEC $06								;\		decrease number of frames drawn...
			RTS									;/		and return
			
XDISPFLAIL:
			db $E6,$E6,$E6,$E4					;		X-displacements when flailing, facing right

XDISPFLAIL2:
			db $1A,$1A,$1A,$1C					;		X-displacements when flailing, facing left
			
YDISPFLAIL:
			db $FA,$F8,$FA,$FD					;		Y-displacements when flailing
			
FLAIL_EXTRA:			
			LDA !FRAMES,x						;\		load frame counter
			TAX									;/		and transfer it to X

			LDA $08								;\		load sprite direction
			BNE +								;/		if facing left, use a different table
			
			LDA $00								;\		take the X-position from GET_DRAW_INFO
			CLC									; |
			ADC XDISPFLAIL,x					; |		add in X-displacements
			STA $0300,y							;/		and store it to the X-position
			BRA ++								;		skip next code
			
+		
			LDA $00								;\		take the X-position from GET_DRAW_INFO
			CLC									; |
			ADC XDISPFLAIL2,x					; |		add in X-displacements
			STA $0300,y							;/		and store it to the X-position

++			
			LDA $01								;\		take the Y-position from GET_DRAW_INFO
			CLC									; |
			ADC YDISPFLAIL,x					; |		add in Y-displacements
			STA $0301,y							;/		and store it to the Y-position
			
			LDA #$80							;\		store the tile number
			STA $0302,y							;/
			
			LDX $15E9							;\		load sprite index, again
			
			LDA $15F6,x							; |		get ccct values
			
			PHX									; |		preserve X
			LDX $08								; |		load sprite direction
			ORA Properties,x					; |		and apply x-flip if needed
			PLX									; |		pull X back
			
			ORA $64								; |		add in level stuff
			STA $0303,y							;/		store to properties
			
			RTS									;		return
			
RUN_EXTRA:			
			LDA $08								;\		load sprite direction
			BNE +								;/		if facing left, use a different table
			
			LDA $00								;\		load X-position from GET_DRAW_INFO
			CLC									; |
			ADC #$E4							; |		add in X-displacement
			STA $0300,y							;/		store to X-position
			BRA ++								;		skip next code
			
+		
			LDA $00								;\		load X-position from GET_DRAW_INFO
			CLC									; |
			ADC #$1C							; |		add in X-displacement
			STA $0300,y							;/		store to X-position

++			
			LDA $01								;\		load Y-position from GET_DRAW_INFO
			STA $0301,y							;/		store to Y-position
			
			LDX $15E9							;		load sprite index
			
			LDA !FRAMES,x						;\		if sprite is on the 3rd frame,
			CMP #$02							; |		load a different tile number
			BEQ +								;/
			
			LDA #$8E							;\		set the tile number
			STA $0302,y							;/
			BRA ++								;		skip next code
			
+
			LDA #$A0							;\		set the tile number
			STA $0302,y							;/
			
++			
			LDA $7FAB10,x						;\		if the first extra bit is set,
			AND #$04							; |		then run this below code
			BEQ +								;/		to change the palette
			
			LDA $15F6,x							;\		get regular ccct values
			ORA #$0F							; |		ORA #$0F to change palette to 0F
			BRA ++								;/		and skip the below code so A is not overwritten
			
+
			LDA $15F6,x							;		get ccct values

++			
			PHX									;\		preserve X
			LDX $08								; |		load sprite direction
			ORA Properties,x					; |		and apply x-flip if needed
			PLX									; |		pull back X
			
			ORA $64								; |		add in level stuff
			STA $0303,y							;/		and store to properties
			
			RTS									;		return

XDISPHIT:
			db $FE,$FB,$EA,$E4					;		X-displacements when hit, facing right

XDISPHIT2:
			db $02,$05,$16,$1C					;		X-displacements when hit, facing left
			
YDISPHIT:
			db $E8,$E8,$F5,$FD					;		Y-displacements when hit
			
HIT_EXTRA:			
			LDA !FRAMES,x						;\		load frame counter
			TAX									;/		and transfer it to X

			LDA $08								;\		load sprite direction
			BNE +								;/		if facing left, use a different table
			
			LDA $00								;\		take the X-position from GET_DRAW_INFO
			CLC									; |
			ADC XDISPHIT,x						; |		add in X-displacements
			STA $0300,y							;/		and store it to the X-position
			BRA ++								;		skip next code
			
+		
			LDA $00								;\		take the X-position from GET_DRAW_INFO
			CLC									; |
			ADC XDISPHIT2,x						; |		add in X-displacements
			STA $0300,y							;/		and store it to the X-position

++			
			LDA $01								;\		take Y-position from GET_DRAW_INFO
			CLC									; |
			ADC YDISPHIT,x						; |		add in Y-displacements
			STA $0301,y							;/		and store it to the Y-position
			
			PHX									;\		preserve X
			LDX $15E9							;/		load sprite index
			
			LDA !FRAMES,x						;\		load the current frame
			PLX									; |		pull back X
			BEQ +								; |		if zero, branch
			CMP #$01							; |		if one, branch
			BEQ ++								;/
			
			LDA #$80							;\		set the tile
			STA $0302,y							;/
			BRA +++								;		skip next code
			
+
			LDA #$A4							;\		set the tile
			STA $0302,y							;/
			BRA +++								;		skip next code

++
			LDA #$AE							;\		set the tile
			STA $0302,y							;/
			
+++			
			LDX $15E9							;		load sprite index
			
			LDA $15F6,x							;		load ccct values
			
			PHX									;\		preserve X
			LDX $08								; |		load sprite direction
			ORA Properties,x					; |		apply X-flip if needed
			PLX									;/		pull back X
			
			ORA $64								;\		add in level stuff
			STA $0303,y							;/		and store to properties
			
			RTS									;		return
			
XDISPGETUP:
			db $E4,$00,$F6,$FC					;\		X-displacements when getting up, facing right
			db $E4,$E4							;/
			
XDISPGETUP2:
			db $1C,$19,$0A,$04					;\		X-displacements when getting up, facing left
			db $1C,$1C							;/
			
YDISPGETUP:
			db $F8,$F8,$E8,$E8					;\		Y-displacements when getting up
			db $E9,$E9							;/
			
GET_UP_EXTRA:			
			LDA !FRAMES,x						;\		load frame counter
			TAX									;/		and transfer it to X

			LDA $08								;\		load sprite direction
			BNE +								;/		if facing left, use a different table
			
			LDA $00								;\		take the X-position from GET_DRAW_INFO
			CLC									; |
			ADC XDISPGETUP,x					; |		add in X-displacements
			STA $0300,y							;/		and store it to the X-position
			BRA ++								;		skip next code
			
+		
			LDA $00								;\		take the X-position from GET_DRAW_INFO
			CLC									; |
			ADC XDISPGETUP2,x					; |		add in X-displacements
			STA $0300,y							;/		and store it to the X-position

++			
			LDA $01								;\		take Y-position from GET_DRAW_INFO
			CLC									; |
			ADC YDISPGETUP,x					; |		add in Y-displacements
			STA $0301,y							;/		and store it to the Y-position
			
			PHX									;\		preserve X
			LDX $15E9							;/		load sprite index
			
			LDA !FRAMES,x						;\		load sprite frame counter
			PLX									;/		pull back X
			BEQ ASD								;		if zero, branch
			
			CMP #$01							;\		if one, branch
			BEQ RAFKJ							;/
			
			CMP #$02							;\		if two, branch
			BEQ sf								;/
			
			CMP #$03							;\		if three, branch
			BEQ AA								;/
			
			LDA #$88							;\		store tile number
			STA $0302,y							;/
			BRA asd								;		skip next code

AA:
			LDA #$AC							;\		store tile number
			STA $0302,y							;/
			BRA asd								;		skip next code
			
ASD:
			LDA #$82							;\		store tile number
			STA $0302,y							;/
			BRA asd								;		skip next code

sf:
			LDA #$8A							;\		store tile number
			STA $0302,y							;/
			
asd:		
			LDX $15E9							;		load sprite index
			
			LDA $15F6,x							;		get ccct values
			
			PHX									;\		preserve X
			LDX $08								; |		load sprite direction
			ORA Properties,x					; |		apply X-flip if needed
			PLX									;/		pull back X
			
			ORA $64								;\		add in level stuff
			STA $0303,y							;/		store to properties
			RTS									;		return
			
RAFKJ:
			DEC $06								;\		decrement $06, as it is the number of tiles drawn
			RTS									;/		return

SHELL_EXTRA:
			LDA !STATE,x						;\		if sprite state is 4,
			CMP #$04							;/		branch
			BEQ +								;

			LDA !FRAMES,x						;\		if frame counter is not at 0,
			BNE RAFKJ							;/		return
			BRA ++								;		skip next code
			
+
			LDA !FRAMES,x						;\		if frame counter is not at 3,
			CMP #$03							;/		return
			BNE RAFKJ							;
			
++			
			LDA $00								;\		take the X-position from GET_DRAW_INFO
			STA $0300,y							;/		and store it to the X-position
			
			LDA $01								;\		take the Y-position from GET_DRAW_INFO
			SEC									; |
			SBC #$13							; |		add in Y-displacement
			STA $0301,y							;/		store to Y-position
			
			LDA #$A4							;\		set the tile number
			STA $0302,y							;/
			
			LDX $15E9							;		load sprite index
			
			LDA $15F6,x							;		get ccct values
			
			PHX									;\		preserve X
			LDX $08								; |		load sprite direction
			ORA Properties,x					; |		apply X-flip if needed
			PLX									;/ 		pull back X
			
			ORA $64								;\		add in level stuff
			STA $0303,y							;/		store to properties
			
			RTS									;		return
			
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Dynamic sprite routine
; Programmed mainly by SMKDan
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SLOTS: 
			db $CC,$C8,$C4,$C0					;		avaliable slots.  Any more transfers and it's overflowing by a dangerous amount.

GETSLOT:
			PHY									;\		preserve OAM index
			PHA									; |		preserve frame
			LDA !SLOTSUSED						;/		test if slotsused == maximum allowed
			CMP #!MAXSLOTS
			BEQ NoneFree

			PLA									;\		pop frame
			REP #$20							; |		16bit A
			AND.w #$00FF						; |		wipe high
			XBA									; |		<< 8
			LSR A								; |		>> 1 = << 7
			STA !TEMP							;/		back to scratch
			LDA GFXPointer
			CLC
			ADC !TEMP							;\		add frame offset	
			STA !SLOTPTR						; |		store to pointer to be used at transfer time
			SEP #$20							; |		8bit store
			PHB									; |		push db...
			PLA									; |		pull to A...
			STA !SLOTBANK						;/		store bank to 24bit pointer

			LDY !SLOTSUSED						;\		calculate VRAM address + tile number
			LDA SLOTS,y							; |		get tile# in VRAM
			PHA									;/		preserve for eventual pull
			SEC
			SBC #$C0							;\		starts at C0h, they start at C0 in tilemap
			REP #$20							; |		16bit math
			AND.w #$00FF						; |		wipe high byte
			ASL A								;/		multiply by 32, since 32 bytes/16 words equates to 1 32bytes tile
			ASL A
			ASL A
			ASL A
			ASL A
			CLC
			ADC #$0B44							;\		add 0B44, base address of buffer	
			STA !SLOTDEST						;/		destination address in the buffer

			JSR DMABUFFER						;		ROM -> RAM copy

			SEP #$20							;\		8bit A	
			INC !SLOTSUSED						;/		one extra slot has been used

			PLA									;		return starting tile number
			PLY
			RTS
		
NoneFree:
			PLA
			PLY
			LDA #$00							;		zero on no free slots
			RTS

;;;;;;;;;;;;;;;;
;Tansfer routine
;;;;;;;;;;;;;;;;

;DMA ROM -> RAM ROUTINE

DMABUFFER:
												;		set destination RAM address
			REP #$20
			LDA !SLOTDEST
			STA $2181							;		16bit RAM dest
			LDY #$7F
			STY $2183							;		set 7F as bank

												;		common DMA settings
			STZ $4300							;\		1 reg only
			LDY #$80							;/		to 2180, RAM write/read
			STY $4301

												;		first line
			LDA !SLOTPTR
			STA $4302							;		low 16bits
			LDY !SLOTBANK
			STY $4304							;\		bank
			LDY #$80							;/		128 bytes
			STY $4305
			LDY #$01
			STY $420B							;		transfer

												;		second line
			LDA !SLOTDEST						;		update buffer dest
			CLC
			ADC #$0200							;\		512 byte rule for sprites
			STA !SLOTDEST						; |		updated base
			STA $2181							;/		updated RAM address

			LDA !SLOTPTR						;		update source address
			CLC
			ADC #$0200							;		512 bytes, next row
			STA !SLOTPTR
			STA $4302							;		low 16bits
			LDY !SLOTBANK
			STY $4304							;		bank
			LDY #$80
			STY $4305
			LDY #$01
			STY $420B							;		transfer

												;		third line
			LDA !SLOTDEST						;		update buffer dest
			CLC
			ADC #$0200							;\		512 byte rule for sprites
			STA !SLOTDEST						; |		updated base
			STA $2181							;/		updated RAM address

			LDA !SLOTPTR						;		update
			CLC
			ADC #$0200
			STA !SLOTPTR
			STA $4302
			LDY !SLOTBANK
			STY $4304
			LDY #$80
			STY $4305
			LDY #$01
			STY $420B							;		transfer

												;		fourth line
			LDA !SLOTDEST						;		update buffer dest
			CLC
			ADC #$0200							;\		512 byte rule for sprites
			STA !SLOTDEST						; |		updated base
			STA $2181							;/		updated RAM address

			LDA !SLOTPTR
			CLC
			ADC #$0200
			STA !SLOTPTR
			STA $4302
			LDY !SLOTBANK
			STY $4304
			LDY #$80
			STY $4305
			LDY #$01
			STY $420B

			SEP #$20							;\		8bit A
			RTS									;/		all done, return

Idle:
			JSL $01802A

			LDA #$2A							;\		make sprite clipping 16x32
			STA $1662,x							;/

			JSR PROXIMITY						;		check if mario is close enough

			STZ $B6,x							;		zero out X-speed

			LDA $14								;\		if frame counter is not an interval of 8,
			AND #$07							; |		don't update image
			BNE DontUpdateImgI					;/		
			
			INC !FRAMES,x						;		increment the current frame
			
			LDA !FRAMES,x						;\		if the current frame is not 0C...
			CMP #$0C							; |		don't zero it
			BNE DontUpdateImgI					;/

			STZ !FRAMES,x						;		zero out the current frame
			
DontUpdateImgI:
			JSL $01ACF9							;		random number generator
			
			LDA $148C							;\		if high byte of result is 0,
			BEQ StartWalking					;/		start walking
			
			AND #$7F							;\		if the 7th bit of the high byte is 1,
			BNE +								;/		change direction
			
			LDA $157C,x							;\		change the direction
			EOR #$01							; |
			STA $157C,x							;/
			
+			
			RTS									;		return
			
StartWalking:
			STZ !FRAMES,x						;		zero out the current frame

			INC !STATE,x						;\		increment sprite state
			RTS									;/		to start walking
			
Walk:
			LDA #$2A							;\		set sprite clipping
			STA $1662,x							;/		to 16x32

			LDA $14								;\		if frame counter is not an interval of 8
			AND #$07							; |		don't update the image
			BNE DontUpdateImgW					;/
			
			INC !FRAMES,x						;		increment the current frame
			
			LDA !FRAMES,x						;\		load the the current frame
			AND #$07							; |		clear out everything except bit 0, 1, and 2
			STA !FRAMES,x						;/
			
DontUpdateImgW:	
			JSL $019138							;		object interaction routine
			
			JSR PROXIMITY						;		check if mario is close enough
			
			LDY $157C,x							;		load the sprite direction into Y
			
			LDA $B6,x							;\		load the X-speed
			CMP XSPEEDTABLE,y					; |		compare it to the max X-speeds
			BEQ +								; |		if it is at the maximum,
												;/		skip acceleration
			
			CLC									;\	
			ADC XACCELTABLE,y					; |		add in the acceleration factor
			STA $B6,x							;/		and store back into the X-speed
			BRA ++
			
+
			LDA XSPEEDTABLE,y					;\		store the X-speed
			STA $B6,x							;/
			
++
			JSL $01802A							;		update sprite X and Y position with gravity

			LDA $1588,x							;\		if touching a wall,
			AND #$03							; |		keep running this code
			BEQ +								;/		branch
			
			LDA $157C,x							;\		flip sprite direction
			EOR #$01							; |		
			STA $157C,x							;/
			
			LDY $157C,x							;		reset Y

			STZ $B6,x							;		zero out X-speed
			
+
			JSL $01ACF9							;		random number generator
			
			LDA $148B							;\		if the low byte of the result is 0,
			BEQ StartIdling						; |		start idling
			RTS									;/		return
			
StartIdling:
			DEC !STATE,x						;\		decrement sprite state
			RTS									;/		to start idling
			
Charge:
			LDA $14								;\		load sprite frame counter
			AND #$03							; |		if it is not an interval of 4,
			BNE DontUpdateImgC					;/		don't updat the image
			
			INC !FRAMES,x						;		increment the current frame
			
			LDA !FRAMES,x						;\		load the current frame
			AND #$03							; |		clear out everything except for bit 0 and 1
			STA !FRAMES,x						;/
			
DontUpdateImgC:
			LDA $1602,x							;\		if sprite direction already transferred,
			BNE +								;/		skip

			JSR SUB_HORZ_POS					;\		make sprite
			TYA									; |		face mario
			STA $157C,x							;/
			
			INC $1602,x							;		increment the sprite direction table
			
+
			LDA $7FAB10,x						;\		take the custom sprite properties
			AND #$04							; |		clear out all bits except for the extra bit
			ASL									; |		double it to 8 if set
			STA $05								;/		and store to some freeram
			
			LDY $157C,x							;\		load sprite direction into Y
			BEQ +								;/		if facing right, go branch
			
			LDA $B6,x							;\		load sprite X-speed
			SEC									; |		we have this check because, otherwise,
			BEQ ++
			
			SBC $05								; |		it would not compare to the maximum speed
			CMP XSPEEDTABLESC,y					; |		compare to max speed
			BCC +++								;/		if it's greater, skip acceleration
			BRA ++								;		skip below code
			
+			
			LDA $B6,x							;\		load sprite X-speed
			SEC									; |		we have this check because, otherwise,
			SBC $05								; |		it would not compare to the maximum speed
			BEQ ++
			CMP XSPEEDTABLESC,y					; |		compare to max speed
			BCS +++								;/		if it's greater, skip acceleration

++			
			CLC									;\	
			ADC XSPEEDTABLESA,y					; |		add acceleration factor
			STA $B6,x							;/		store to X-speed
			
			BRA ++								;		skip below code
			
+++			
			LDA XSPEEDTABLESC,y					;\		load the speed tables
			CLC									; |		and add the value taken earlier; if the extra bit is clear,
			ADC $05								; |		it will still be zero, but, if it is set, it will be eight
			STA $B6,x							;/		store to the X-speed
			
			
++			
			JSL $01802A							;		update sprite X and Y position with gravity
			
			LDA $1588,x							;\		if the sprite is touching a wall,
			AND #$03							; |		run this code
			BEQ +								;/		if not, skip this
			
			LDA #$18							;\		load how long to stay at the next sprite state
			STA !SHELLTIMER,x					;/		store to a decreasing table
			
			LDY $157C,x							;		load the current sprite direction
			
			LDA XSPEEDTABLEHIT,y				;\		load the knockback X-speeds
			STA $B6,x							;/		and store it to the sprite's X-speed
			
			LDA #$E0							;\		set the sprite's Y-speed
			STA $AA,x							;/
			
			STZ !FRAMES,x						;		zero out the current frame
			
			STZ $1602,x							;		zero out the sprite table
			
			JSL $01802A							;		update sprite X and Y position with gravity
			
			LDA #$08							;\		play the spin jump kill sound
			STA $1DF9							;/		
			
			INC !STATE,x						;		increment the sprite state
			
+			
			RTS		
			
ShellSpin:
			LDA $1602,x							;\		if sprite direction already transferred,
			BNE +								;/		skip

			JSR SUB_HORZ_POS					;\		make sprite
			TYA									; |		face mario
			STA $157C,x							;/
			
			INC $1602,x							;		increment the sprite direction table
			
+			
			STZ $1662,x							;		make sprite clipping 16x16
			
			LDA $1588,x							;\		if in the air already,
			AND #$04							; |		do not pop out of shell
			BEQ +								;/		since it looks weird
			
			LDA !SHELLTIMER,x					;\		if the shell timer has finished,
			BEQ ChangeStateSS					;/		change the state

+			
			LDA $14								;\		if sprite frame counter is not an interval of 4,
			AND #$03							; |		don't update the image
			BNE +								;/
			
			INC !FRAMES,x						;		increment the current frame
			
			LDA !FRAMES,x						;\		if the current frame is 0C,
			CMP #$0C							; |
			BNE +								;/
			
			STZ !FRAMES,x						;		zero it out
			
+
			LDA $7FAB10,x						;\		take the custom sprite properties
			AND #$04							; |		clear out all bits except for the extra bit
			ASL									; |		double it to 8 if set
			STA $05								;/		and store to some freeram
			
			LDY $157C,x							;\		load sprite direction into Y
			BEQ +								;/		if facing right, go branch
			
			LDA $B6,x							;\		load sprite X-speed
			SEC									; |		we have this check because, otherwise,
			SBC $05								; |		it would not compare to the maximum speed
			BEQ ++
			
			CMP XSPEEDTABLESS,y					; |		compare to max speed
			BCC +++								;/		if it's greater, skip acceleration
			BRA ++								;		skip below code
			
+			
			LDA $B6,x							;\		load sprite X-speed
			SEC									; |		we have this check because, otherwise,
			SBC $05								; |		it would not compare to the maximum speed
			BEQ ++
			
			CMP XSPEEDTABLESS,y					; |		compare to max speed
			BCS +++								;/		if it's greater, skip acceleration

++			
			CLC									;\	
			ADC XSPEEDTABLESAC,y				; |		add acceleration factor
			STA $B6,x							;/		store to X-speed
			
			BRA ++								;		skip below code
			
+++	
-		
			LDA XSPEEDTABLESS,y					;\		load the speed tables
			CLC									; |		and add the value taken earlier; if the extra bit is clear,
			ADC $05								; |		it will still be zero, but, if it is set, it will be eight
			STA $B6,x							;/		store to the X-speed
			
			
++						
			JSL $01802A							;		update sprite X and Y position with gravity
			
			LDA $1588,x							;\		if sprite is touching a wall,
			AND #$03							; |		run this code
			BEQ +								;/ 		if not, skip it
			
			LDA $157C,x							;\		flip sprite direction
			EOR #$01							; |
			STA $157C,x							;/	
			
			LDA #$01							;\		and play a sound
			STA $1DF9							; |
			STA !MAXSPEED,x						;/		and set max speed flag
			
			BRA -								;		and branch backwards
			
+					
			RTS									;		return
			
ChangeStateSS:
			LDA #$0C							;\		store 0C
			STA !SHELLTIMER,x					; |		to the sprite timer
												;/		as it uses it to figure out when to change states

			STZ $1602,x							;		zero out the sprite table									
												
			STZ !FRAMES,x						;		zero out the current frame

			INC !STATE,x						;\		increment the sprite state
			RTS									;/		so that it will go out of its shell
			
PROXIMITY:
			LDA $1490
			BEQ +
			
			RTS

+			
			JSR SUB_HORZ_POS					;		jsr to sub horz pos; $0F isn't used though

			LDA $E4,x							;\		take low byte of sprite X-pos
			STA $00								;/		and store to scratch RAM
			
			LDA $14E0,x							;\		take high byte of sprite X-pos
			STA $01								;/		and store to scratch RAM
			
			LDA $157C,x							;\		if sprite is facing right,
			BEQ RIGHT							;/		branch
			
			CPY #$00							;\		if mario is to the right of the sprite,
			BEQ GASdf							;/		branch
			
			REP #$20							;		16-bit math
			
			LDA $00								;\		take the sprite x-pos
			SEC									; |		and subtract
			SBC $94								;/		Mario's X-pos
			
			CMP #$0040							;\		compare it to 0040
			BCS GASdf							;/		if it is greater, return
			
			SEP #$20							;		8-bit math
			
			BRA GO								;		run code now
			
RIGHT:
			CPY #$01							;\		if mario is to the left of the sprite,
			BEQ GASdf							;/		branch

			REP #$20							;		16-bit math
			
			LDA $94								;\		take mario's x-pos
			SEC									; |		and subtract
			SBC $00								;/		the sprite's X-pos
			
			CMP #$0040							;\		compare it to 0040
			BCS GASdf							;/		if it is greater, return
			
			SEP #$20							;		8-bit math
			
GO:
			JSR SUB_VERT_POS					;\ 		if sprite closer to Mario than 2 blocks,
			
			LDA $0E								; |
			CLC									; |
			ADC #$20							; |
			CMP #$40							; |
			BCS GASdf							;/		return

			JSL $01ACF9							;		random number generator
			
			LDA $148B							;\		if the low byte of the result
			AND #$03							; |		is NOT divisible by 4
			BNE ASFS							;/		then make the koopatrol charge at you
			
			LDA !DISABLESHELLTIMER,x			;\		if the timer to disable the shell is still active
			BNE GASdf							;/		return

			LDA !STATE,x						;\		if the sprite state is 01 (walking)
			CMP #$01							; |		
			BEQ +								;/		branch, so that nothing glitches up
			
			INC !STATE,x						;		increment sprite state
+			
			INC !STATE,x						;		increment sprite state
			
			STZ !FRAMES,x						;		zero out the current frame
			
			LDA #$0C							;\		load 0C
			STA !SHELLTIMER,x					;/		and store it to how long the next sprite state (going in shell) will last

			JSR SUB_HORZ_POS					;\		make sprite
			TYA									; |		face mario
			STA $157C,x							;/		
			
GASdf:			
			SEP #$20

			RTS									;		return
			
ASFS:			
			LDA !STATE,x						;\		if sprite state is 01 (walking)
			CMP #$01							; |		
			BEQ +								;/		branch, so that nothing glitches up
			
			INC !STATE,x						;		increment sprite state
+			
			REP 4 : INC !STATE,x				;		increment sprite state 4 times
			
			STZ !FRAMES,x						;		zero out the current frame
			
			LDA #$0C							;\		load 0C
			STA !SHELLTIMER,x					;/		and store it to how long the next sprite state (going in shell) will last
			RTS									;		return
			
GoingInShell:
			LDA !SHELLTIMER,x					;\		if the shell timer is finished,
			BEQ ChangeStateG					;/		change the state to shell spinning

			LDA $14								;\		if sprite frame counter is not an interval of 4
			AND #$03							; |		don't update the image
			BNE DontUpdateImgG					;/
			
			INC !FRAMES,x						;		increment the current frame
			
			LDA !FRAMES,x						;\		load the current frame
			AND #$03							; |		clear out all bits except bit 0 and 1
			STA !FRAMES,x						;/
			
DontUpdateImgG:
			RTS									;		return
			
ChangeStateG:
			LDA #$50							;\		load how long the koopatrol should stay in its shell
			STA !SHELLTIMER,x					;/		and store it to a timer

			STZ !FRAMES,x						;		zero out the current frame

			INC !STATE,x						;\		increment the sprite state
			RTS									;/		to make it go in its shell
			
GettingOutOfShell:
			LDA !SHELLTIMER,x					;\		if the shell timer is finished,
			BEQ ChangeStateGOS					;/		change the state to idling
			
			LDA $14								;\		if sprite frame counter is not an interval of 4
			AND #$03							; |		don't update the image
			BNE DontUpdateImgGOS				;/
			
			INC !FRAMES,x						;		increment the current frame
			
			LDA !FRAMES,x						;\		load the current frame
			AND #$03							; |		clear out all bits except bit 0 and 1
			STA !FRAMES,x						;/
			
DontUpdateImgGOS:		
			RTS									;		return
			
ChangeStateGOS:
			LDA #$60							;\		load how long to disable shell spinning
			STA !DISABLESHELLTIMER,x			;/		and store it

			STZ !FRAMES,x						;\		zero out the current frame
			STZ !STATE,x						;/		and the current sprite state
			
			RTS									;		return
			
HitWall:
			JSL $01802A							;		update sprite X and Y position with gravity

			LDA !SHELLTIMER,x					;\		if the timer is finished...
			BEQ ChangeStateH					;/		change the state to flailing

			LDA $14								;\		if sprite frame counter is not an interval of 8,
			AND #$07							; |		don't update the image
			BNE DontUpdateImgH					;/		
			
			INC !FRAMES,x						;		increment the current frame
			
			LDA !FRAMES,x						;\		load the current frame
			AND #$03							; |		clear out all bits except bit 0 and 1
			STA !FRAMES,x						;/
			
DontUpdateImgH:			
			RTS									;		return
			
ChangeStateH:
			LDA #$50							;\		load the time
			STA !SHELLTIMER,x					;/		to keep the koopatrol on its back

			STZ !FRAMES,x						;		zero the current frame
			
			INC !STATE,x						;\		and increment the sprite state
			RTS									;/		to make it flail

Flail:
			LDA $1588,x							;\		if on the ground,
			AND #$04							; |
			BNE +								;/		do not update positions
			
			JSL $01802A							;		update positions with gravity	

+			
			STZ $1662,x							;		make the sprite clipping 16x16

			LDA !SHELLTIMER,x					;\		if the timer is finished,
			BEQ ChangeStateF					;/		change the state to getting up
			
			LDA $14								;\		if sprite frame counter is not an interval of 4
			AND #$03							; |		don't update the image
			BNE DontUpdateImgF					;/
			
			INC !FRAMES,x						;		increment the current frame
			
			LDA !FRAMES,x						;\		load the current frame
			AND #$03							; |		clear out all bits except bit 0 and 1
			STA !FRAMES,x						;/
			
DontUpdateImgF:			
			RTS									;		return
			
ChangeStateF:
			LDA #$18							;\		load the time
			STA !SHELLTIMER,x					;/		to change to the next state after this

			STZ !FRAMES,x						;		zero out the current frame
			
			INC !STATE,x						;\		increment the sprite state
			RTS									;/		so it starts getting up
			
GetUp:
			LDA !SHELLTIMER,x					;\		if timer is finished
			BEQ ChangeStateGU					;/		then change the state to idling

			LDA $14								;\		if sprite frame counter is not an interval of 8,
			AND #$07							; |		don't update the image
			BNE DontUpdateImgGU					;/		
			
			INC !FRAMES,x						;		increment the current frame
			
DontUpdateImgGU:
			RTS									;		return
			
ChangeStateGU:
			STZ !FRAMES,x						;		zero out the current frame

			STZ !STATE,x						;\		zero out the sprite state
			RTS									;/		completely done, return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B75C

TABLE1:              db $0C,$1C
TABLE2:              db $01,$02

GET_DRAW_INFO:
                    STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
					LDA $E4,x               ; \
					CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X:
                    LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC #$0040              ;  |
                    CMP #$0180              ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP:
                    LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC TABLE1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA TABLE2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y:
                    DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID:
                    PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /

SPRITE_INTERACT:    LDY #$0B                ; 
LOOP:
			LDA $14C8,y             ; \ if the sprite status is..
			CMP #$09                ;  | ...shell-like
			BEQ PROCESS_SPRITE      ; /
			CMP #$0A                ; \ ...throwned shell-like
			BEQ PROCESS_SPRITE      ; /
			
NEXT_SPRITE:
			DEY                     ;
			BPL LOOP                ; ...otherwise, loop
			RTS                     ; return

PROCESS_SPRITE:
			PHX                     ; push X
			TYX                     ; transfer Y to X
			JSL $03B6E5             ; get sprite clipping B routine
			PLX                     ; pull X
			JSL $03B69F             ; get sprite clipping A routine
			JSL $03B72B             ; check for contact routine
			BCC NEXT_SPRITE         ;

			PHX                     ; push X
			TYX                     ; transfer Y to X

			STZ $14C8,x             ; destroy the sprite


BLOCK_SETUP:
			LDA $E4,x               ;\ setup block properties
			STA $9A                 ; |
			LDA $14E0,x             ; |
			STA $9B                 ; |
			LDA $D8,x               ; |
			STA $98                 ; |
			LDA $14D4,x             ; |
			STA $99                 ;/

EXPLODING_BLOCK:
			PHB                     ;\ set the exploding block routine
			LDA #$02                ; |
			PHA                     ; |
			PLB                     ; |
			LDA #$FF                ; | $FF = set flashing palette
			JSR $028663             ; |

			PLA						;/
			PLX
					
			RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SPR_T12:             db $40,$B0
SPR_T13:             db $01,$FF
SPR_T14:             db $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
		            db $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
SPR_T15:             db $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
					db $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SUB_OFF_SCREEN_X1:   LDA #$02                ; \ entry point of routine determines value of $03
                     BRA STORE_03            ;  | (table entry to use on horizontal levels)
SUB_OFF_SCREEN_X2:   LDA #$04                ;  | 
                     BRA STORE_03            ;  |
SUB_OFF_SCREEN_X3:   LDA #$06                ;  |
                     BRA STORE_03            ;  |
SUB_OFF_SCREEN_X4:   LDA #$08                ;  |
                     BRA STORE_03            ;  |
SUB_OFF_SCREEN_X5:   LDA #$0A                ;  |
                     BRA STORE_03            ;  |
SUB_OFF_SCREEN_X6:   LDA #$0C                ;  |
                     BRA STORE_03            ;  |
SUB_OFF_SCREEN_X7:   LDA #$0E                ;  |
STORE_03:			STA $03					;  |            
					BRA START_SUB			;  |
SUB_OFF_SCREEN_X0:   STZ $03					; /

START_SUB:           JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ; | 
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; | 
                    CMP #$02                ; | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31:             LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE:        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /    
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE:         STZ $14C8,x             ; erase sprite
RETURN_35:           RTS                     ; return

VERTICAL_LEVEL:      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; | 
                    BCS RETURN_35           ; /
                    LDA $E4,x               ; \ 
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA $001D              ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38:            LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490

SUB_IS_OFF_SCREEN:   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ; |  
                    RTS                     ; / return
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB HORZ POS
;
; $B817 - horizontal mario/sprite check - shared
; Y = 1 if mario left of sprite??
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B817        

SUB_HORZ_POS:        LDY #$00                ;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
                    LDA $94                 ;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
                    SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
                    SBC $E4,x               ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
                    STA $0F                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
                    LDA $95                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
                    SBC $14E0,x             ;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
                    BPL LABEL16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
                    INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
LABEL16:             RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642

SUB_VERT_POS:
     		LDY #$00
			LDA $D3
			SEC
			SBC $D8,x
			STA $0E
			LDA $D4
			SBC $14D4,x
			BPL Return01AD53
			INY
Return01AD53:		RTS					; return

STAR_SOUNDS:         	db $00,$13,$14,$15,$16,$17,$18,$19

SUB_STOMP_PTS:      	 PHY                     ; 
					LDA $1697               ; \
					CLC                     ;  | 
					ADC $1626,x             ; / some enemies give higher pts/1ups quicker??
					INC $1697               ; increase consecutive enemies stomped
					TAY                     ;
					INY                     ;
					CPY #$08                ; \ if consecutive enemies stomped >= 8 ...
					BCS NO_SOUND            ; /    ... don't play sound 
					LDA STAR_SOUNDS,y       ; \ play sound effect
					STA $1DF9               ; /   
NO_SOUND:
					TYA                     ; \	
					CMP #$08                ;  | if consecutive enemies stomped >= 8, reset to 8
					BCC NO_RESET            ;  |
					LDA #$08                ; /
NO_RESET:
					JSL $02ACE5             ; give mario points
					PLY                     ;
					RTS                     ; return

KILLED_X_SPEED:      db $F0,$10
HAS_STAR:
                    LDA #$D0                ; \ set y speed
                    STA $AA,x               ; /
                    JSR SUB_HORZ_POS        ; get new direction
                    LDA KILLED_X_SPEED,y    ; \ set x speed based on direction
                    STA $B6,x               ; /
                    INC $18D2               ; increment number consecutive enemies killed
                    LDA $18D2               ; \
                    CMP #$08                ;  | if consecutive enemies stomped >= 8, reset to 8
                    BCC NO_RESET2           ;  |
                    LDA #$08                ;  |
                    STA $18D2               ; /   
NO_RESET2:           JSL $02ACE5             ; give mario points
                    LDY $18D2               ; \ 
                    CPY #$08                ;  | if consecutive enemies stomped < 8 ...
                    BCS NO_SOUND2           ;  |
                    LDA STAR_SOUNDS,y       ;  |    ... play sound effect
                    STA $1DF9               ; /
NO_SOUND2:           RTS                     ; final return					
					
GFXPointer:
			dw GFX
GFX:
incbin sprites\koopatrol.bin			;included graphics file